import sys
import unittest2 as unittest
import socket
import inspect

from raygun4py import utilities

class TestRaygunUtilities(unittest.TestCase):
    def test_filter_keys(self):
        test_obj = {
            'foo': 'bar',
            'baz': 'qux'
        }

        utilities.filter_keys(['foo'], test_obj)

        self.assertEqual(test_obj['foo'], '<filtered>')

    def test_filter_keys_recursive(self):
        test_obj = {
            'foo': 'bar',
            'baz': 'qux',
            'boo': {
                'foo': 'qux'
            }
        }

        utilities.filter_keys(['foo'], test_obj)

        self.assertEqual(test_obj['foo'], '<filtered>')
        self.assertEqual(test_obj['boo']['foo'], '<filtered>')


    def test_filter_keys_with_wildcard(self):
        test_obj = {
            'foobr': 'bar',
            'foobz': 'baz',
            'fooqx': 'foo',
            'baz': 'qux'
        }

        utilities.filter_keys(['foo*'], test_obj)

        self.assertEqual(test_obj['foobr'], '<filtered>')
        self.assertEqual(test_obj['foobz'], '<filtered>')
        self.assertEqual(test_obj['fooqx'], '<filtered>')

def main():
    unittest.main()

if __name__ == '__main__':
    main()





def build_wsgi_compliant_request(request):
    if not request:
        return

    rg_request = None
    # start with WSGI environ variables, then overlay the specific, expected
    # httpRequest keys for RG API compat
    # https://www.python.org/dev/peps/pep-3333/#environ-variables

    http_host = request.get('hostName') \
                or request.get('HTTP_HOST') \
                or "{}:{}".format(request.get('SERVER_NAME'), request.get('SERVER_PORT'))

    if http_host is not None:
        http_host = http_host.replace(' ', '')

    try:
        http_form = getattr(request, 'form', None) or request.get('form')

        # fallback to wsgi.input
        if http_form is None and 'wsgi.input' in request:
            # we can assume WSGI keys inside this block
            content_length = int(request.get('CONTENT_LENGTH', 0))
            if content_length:
                http_form = request['wsgi.input'].read(content_length)

        rg_request = {
            'httpMethod': (request.get('httpMethod') or request.get('REQUEST_METHOD')),
            'url': (request.get('url') or request.get('PATH_INFO')),
            'hostName': http_host,
            'iPAddress': (request.get('ipAddress') or request.get('iPAddress') or request.get('REMOTE_ADDR')),
            'queryString': (request.get('queryString') or request.get('QUERY_STRING')),
            'headers': {}, # see below
            'form': http_form,
            'rawData': request.get('rawData')
        }
    except Exception:
        pass

    # Header processing
    _headers = request.get('headers')
    if _headers is None:
        # manually try to build up headers given known WSGI keys
        _headers = {
            'Content-Type': request.get('CONTENT_TYPE'),
            'Content-Length': request.get('CONTENT_LENGTH'),
        }

        for key, value in request.items():
            if key.startswith('HTTP_'):
                # 'HTTP_REFERER' => 'Referer'
                new_key = http_environ_var_to_header_key(key)
                _headers[new_key] = value

    rg_request['headers'] = _headers

    return rg_request


def http_environ_var_to_header_key(key):
    "turns HTTP_REFERER like keys into Referer"
    parts = key.split('_')
    if parts[0] == "HTTP":
        parts.pop(0)

    return '-'.join([x.title() for x in parts])


import logging

import flask
from flask.signals import got_request_exception

from raygun4py import raygunprovider

log = logging.getLogger(__name__)


class Provider(object):
    def __init__(self, flaskApp, apiKey, config=None):
        self.flaskApp = flaskApp
        self.apiKey = apiKey
        self.config = config if config is not None else {}
        self.sender = None

        got_request_exception.connect(self.send_exception, sender=flaskApp)

        flaskApp.extensions['raygun'] = self

    def attach(self):
        if not hasattr(self.flaskApp, 'extensions'):
            self.flaskApp.extensions = {}

        self.sender = raygunprovider.RaygunSender(
            self.apiKey, config=self.config)
        return self.sender

    def send_exception(self, exception=None, exc_info=None, user=None, **kwargs):
        """
        Send an exception report to Raygun. This middleware method enhances the report with Flask-specific environment data before sending.

        Parameters:
            exception (Exception, optional): An exception instance to report.
            exc_info (tuple, optional): A 3-tuple containing exception type, exception instance, and traceback.
            user (dict or str, optional): Information about the affected user.
            tags (list, optional): A list of tags relating to the current context which you can define.
            userCustomData (dict, optional): A dictionary containing custom key-values also of your choosing.
            httpRequest (dict, optional): HTTP Request data that you wish to include with the report.

        Returns:
            The result of the post request, typically indicating the success or failure of the exception report transmission.
        """
        if not self.sender:
            log.error("Raygun-Flask: cannot send as provider not attached!")
            return

        env = self._get_flask_environment()
        # Ensure extra_environment_data is merged or added
        if 'extra_environment_data' in kwargs:
            kwargs['extra_environment_data'].update(env)
        else:
            kwargs['extra_environment_data'] = env

        self.sender.send_exception(
            exception=exception, exc_info=exc_info, user_override=user, **kwargs)

    def _get_flask_environment(self):
        return {
            'frameworkVersion': 'Flask ' + getattr(flask, '__version__', '')
        }



import copy
import logging
import socket
import sys

import jsonpickle
import requests

from raygun4py import raygunmsgs, utilities

DEFAULT_CONFIG = {
    'before_send_callback': None,
    'grouping_key_callback': None,
    'filtered_keys': [],
    'ignored_exceptions': [],
    'proxy': None,
    'transmit_global_variables': True,
    'transmit_local_variables': True,
    'transmit_environment_variables': True,
    'userversion': "Not defined",
    'user': None,
    'http_timeout': 10.0
}


class RaygunSender:
    """
    A sender for reporting errors to Raygun.

    Attributes:
        api_key (str): The API key for Raygun.
    """
    log = logging.getLogger(__name__)

    api_key = None
    endpointprotocol = 'https://'
    endpointhost = 'api.raygun.io'
    endpointpath = '/entries'
    process_tags = []

    def __init__(self, api_key, config={}):
        """
        Initialize a RaygunSender.

        Parameters:
            api_key (str): The API key for Raygun.
            config (dict, optional): Configuration options. Defaults to an empty dictionary.
        """
        if api_key:
            if not isinstance(api_key, str):
                self.log.error(
                    f"Expected api_key of type str, but got {type(api_key).__name__}. Errors will not be transmitted.")
                self.api_key = None
            else:
                self.api_key = api_key
        else:
            self.log.warning(
                "RaygunProvider error: api_key not set, errors will not be transmitted.")

        try:
            import ssl
        except ImportError:
            self.log.warning("RaygunProvider error: No SSL support available, cannot send. Please"
                             "compile the socket module with SSL support.")

        # Set up the default values
        default_config = utilities.snakecase_dict(
            copy.deepcopy(DEFAULT_CONFIG))
        default_config.update(utilities.snakecase_dict(config or {}))
        for k, v in default_config.items():
            setattr(self, k, v)

    def set_version(self, version):
        """
        Set the version for the error reports.

        Parameters:
            version (str): Version string.
        """
        if isinstance(version, str):
            self.userversion = version

    def set_user(self, user):
        """
        Set user information for the error reports.

        Parameters:
            user: User information.
        """
        self.user = user

    def set_tags(self, tags):
        """
        Set tags for the error reports.

        Parameters:
            tags (list): List of tags.
        """
        if type(tags) is list:
            self.process_tags = tags

    def ignore_exceptions(self, exceptions):
        """
        Set exceptions that should be ignored.

        Parameters:
            exceptions (list): List of exceptions to ignore.
        """
        if isinstance(exceptions, list):
            self.ignored_exceptions = exceptions

    def filter_keys(self, keys):
        """
        Set keys to be filtered out from the error reports.

        Parameters:
            keys (list): List of keys to filter out.
        """
        if isinstance(keys, list):
            self.filtered_keys = keys

    def set_proxy(self, host, port):
        """
        Set a proxy for the sender.

        Parameters:
            host (str): Proxy host.
            port (int): Proxy port.
        """
        self.proxy = {
            'host': host,
            'port': port
        }

    def on_before_send(self, callback):
        """
        Set a callback to be executed before sending a report.

        Parameters:
            callback (callable): Callback function to be executed.
        """
        if callable(callback):
            self.before_send_callback = callback

    def on_grouping_key(self, callback):
        """
        Set a callback to customize the grouping key for the report.

        Parameters:
            callback (callable): Callback function to customize grouping.
        """
        if callable(callback):
            self.grouping_key_callback = callback

    def send_exception(self, exception=None, exc_info=None, user_override=None, **kwargs):
        """
        Send an exception report to Raygun.

        Parameters:
            exception (Exception, optional): An exception instance to report.
            exc_info (tuple, optional): A 3-tuple containing exception type, exception instance, and traceback.
            user_override (dict or str, optional): Information about the affected user. If not provided, 'self.user' will be used.
            tags (list, optional): A list of tags relating to the current context which you can define.
            userCustomData (dict, optional): A dictionary containing custom key-values also of your choosing.
            httpRequest (dict, optional): HTTP Request data that you wish to include with the report.

        Returns:
            The result of the post request, typically indicating the success or failure of the exception report transmission.
        """
        options = {
            'transmitLocalVariables': self.transmit_local_variables,
            'transmitGlobalVariables': self.transmit_global_variables
        }
        tags, custom_data, http_request, extra_environment_data, custom_message, fallback_error = self._parse_args(
            kwargs)

        exc_type, exc_value, exc_traceback = exc_info or sys.exc_info()
        if fallback_error and not (exc_type or exception):
            errorMessage = fallback_error
        else:
            errorMessage = self._create_error_message(
                exception, exc_type, exc_value, exc_traceback, options, custom_message)

        message = self._create_message(
            errorMessage, tags, custom_data, http_request, extra_environment_data, user_override)
        message = self._transform_message(message)

        if message is not None:
            return self._post(message)

    def _create_error_message(self, exception, exc_type, exc_value, exc_traceback, options, custom_message=None):
        if exception:
            return raygunmsgs.RaygunErrorMessage(type(exception), exception, exception.__traceback__, options, custom_message)
        else:
            return raygunmsgs.RaygunErrorMessage(exc_type, exc_value, exc_traceback, options, custom_message)

    def _parse_args(self, kwargs):
        tags = kwargs['tags'] if 'tags' in kwargs else None
        custom_data = kwargs['userCustomData'] if 'userCustomData' in kwargs else None
        extra_environment_data = kwargs['extra_environment_data'] if 'extra_environment_data' in kwargs else None
        custom_message = kwargs['custom_message'] if 'custom_message' in kwargs else None
        fallback_error = kwargs['fallback_error'] if 'fallback_error' in kwargs else None

        http_request = None
        if 'httpRequest' in kwargs:
            http_request = kwargs['httpRequest']
        elif 'request' in kwargs:
            http_request = kwargs['request']

        return tags, custom_data, http_request, extra_environment_data, custom_message, fallback_error

    def _create_message(self, raygunExceptionMessage, tags, user_custom_data, http_request, extra_environment_data, user_override=None):
        options = {
            'transmit_environment_variables': self.transmit_environment_variables
        }
        return raygunmsgs.RaygunMessageBuilder(options).new() \
            .set_machine_name(socket.gethostname()) \
            .set_version(self.userversion) \
            .set_client_details() \
            .set_exception_details(raygunExceptionMessage) \
            .set_environment_details(extra_environment_data) \
            .set_tags(self.process_tags) \
            .set_tags(tags) \
            .set_customdata(user_custom_data) \
            .set_request_details(http_request) \
            .set_user(user_override if user_override else self.user) \
            .build()

    def _transform_message(self, message):
        message = utilities.ignore_exceptions(self.ignored_exceptions, message)

        if message is not None:
            message = utilities.filter_keys(self.filtered_keys, message)
            message['details']['groupingKey'] = utilities.execute_grouping_key(
                self.grouping_key_callback, message)

        if self.before_send_callback is not None:
            mutated_payload = self.before_send_callback(message['details'])

            if mutated_payload is not None:
                message['details'] = mutated_payload
            else:
                return None

        return message

    def _post(self, raygunMessage):
        json = jsonpickle.encode(raygunMessage, unpicklable=False)

        try:
            headers = {
                "X-ApiKey": self.api_key,
                "Content-Type": "application/json",
                "User-Agent": "raygun4py"
            }

            response = requests.post(self.endpointprotocol + self.endpointhost + self.endpointpath,
                                     headers=headers, data=json, timeout=self.http_timeout)
        except Exception as e:
            self.log.error(e)
            return 400, "Exception: Could not send"
        return response.status_code, response.text


class RaygunHandler(logging.Handler):
    """
    Logging handler for sending error logs to Raygun.

    Attributes:
        sender (RaygunSender): The RaygunSender to use for sending.
        version (str): Version for the RaygunSender.
    """

    def __init__(self, api_key=None, version=None, level=logging.ERROR, sender=None):
        """
        Initialize a RaygunHandler for logging.

        For constructing instances using an existing RaygunSender, it is recommended to use the `RaygunHandler.from_sender()` class method.

        Parameters:
            api_key (str): The API key for Raygun.
            version (str, optional): Version for the RaygunSender. Defaults to None.
            level (int, optional): Logging level. Defaults to logging.ERROR.
        """
        super().__init__(level)
        if api_key:
            self.sender = RaygunSender(api_key)
            if version:
                self.sender.set_version(version)
            self.version = version
        elif sender:
            self.sender = sender
            self.version = sender.userversion
        else:
            raise ValueError("Either 'api_key' or 'sender' must be provided.")

    @classmethod
    def from_sender(cls, sender, level=logging.ERROR):
        """
        Construct a RaygunHandler instance using an existing RaygunSender object.

        Parameters:
            sender (RaygunSender): The RaygunSender to use for sending.
            level (int, optional): Logging level. Defaults to logging.ERROR.

        Returns:
            RaygunHandler: A new instance of RaygunHandler.
        """
        if not isinstance(sender, RaygunSender):
            raise TypeError(
                f"Expected sender of type RaygunSender, but got {type(sender).__name__}")
        return cls(level=level, sender=sender)

    def emit(self, record):
        # Use log level as a tag
        tag = self.get_tag_from_levelname(record.levelname)
        tags = [tag] if tag else []

        if record.exc_info:
            # exc_info was provided, so send it
            userCustomData = {
                "Logger Name": record.name,
            }
            self.sender.send_exception(
                exc_info=record.exc_info, userCustomData=userCustomData, tags=tags, custom_message=record.getMessage())
        else:
            # Proide a fallback error to use if exc_info cannot be obtained from sys
            fallback_error = raygunmsgs.RaygunLoggerFallbackErrorMessage(
                record.name, record.getMessage(), record.filename, record.funcName, record.lineno
            )
            self.sender.send_exception(
                tags=tags, fallback_error=fallback_error)

    @staticmethod
    def get_tag_from_levelname(levelname):
        tag_map = {
            'DEBUG': 'Debug Log',
            'INFO': 'Info Log',
            'WARNING': 'Warning Log',
            'ERROR': 'Error Log',
            'CRITICAL': 'Critical Log'
        }
        return tag_map.get(levelname)



import mock
import unittest

from webtest import TestApp
from webtest.debugapp import debug_app
from raygun4py.middleware.wsgi import Provider

class TestWSGIMiddleware(unittest.TestCase):

    def setUp(self):
        self.test_middleware = ExceptionMiddleware(debug_app)
        self.raygun_middleware = Provider(self.test_middleware, "XXXXXXXXXX")
        self.app = TestApp(app=self.raygun_middleware, lint=True)

        self.raygun_middleware.sender.send_exception = mock.MagicMock(return_value=True)

    def test_basic_exception(self):
        with self.assertRaises(Exception):
            self.app.get('/?error=t')

        self.raygun_middleware.sender.send_exception.assert_called_once()

    def test_json_post(self):
        self.test_middleware.raise_on_request = True

        with self.assertRaises(Exception):
            self.app.post_json('/foo/bar', params={
                'foo': 'bar'
            })

        self.raygun_middleware.sender.send_exception.assert_called_once()


class ExceptionMiddleware(object):

    def __init__(self, app):
        self.app = app
        self.raise_on_request = False

    def __call__(self, environ, start_response):
        appiter = None

        try:
            appiter = self.app(environ, start_response)

            if self.raise_on_request:
                raise Exception("test exception")

            for item in appiter:
                yield item
        except Exception:
            raise
        finally:
            if hasattr(appiter, 'close'):
                appiter.close()


import logging

from raygun4py import raygunprovider, http_utilities


log = logging.getLogger(__name__)


class Provider(object):

    def __init__(self, app, apiKey):
        self.app = app
        self.sender = raygunprovider.RaygunSender(apiKey)

    def __call__(self, environ, start_response):
        if not self.sender:
            log.error("Raygun-WSGI: Cannot send as provider not attached")

        iterable = None

        try:
            iterable = self.app(environ, start_response)
            for event in iterable:
                yield event

        except Exception as e:
            try:
                request = http_utilities.build_wsgi_compliant_request(environ)
                self.sender.send_exception(exception=e, request=request)
            except Exception as inner:
                log.error("Raygun-WSGI: could not send exception!")
                log.error(inner)

            raise

        finally:
            if hasattr(iterable, 'close'):
                try:
                    iterable.close()
                except Exception as e:
                    try:
                        rg_request_details = http_utilities.build_wsgi_compliant_request(environ)
                        self.sender.send_exception(exception=e, request=rg_request_details)
                    except Exception as inner:
                        log.error("Raygun-WSGI: could not send exception on close!")
                        log.error(inner)
                    raise




import logging

from raygun4py import http_utilities, raygunprovider

log = logging.getLogger(__name__)


class Provider(object):

    def __init__(self, app, apiKey, config=None):
        self.app = app
        self.config = config if config is not None else {}
        self.sender = raygunprovider.RaygunSender(apiKey, config=self.config)

    def __call__(self, environ, start_response):
        iterable = None
        try:
            iterable = self.app(environ, start_response)
            for event in iterable:
                yield event

        except Exception as e:
            if self.sender:
                try:
                    request = http_utilities.build_wsgi_compliant_request(
                        environ)
                    self.sender.send_exception(exception=e, request=request)
                except Exception as inner:
                    log.error("Raygun-WSGI: could not send exception!")
                    log.error(inner)
            else:
                log.error("Raygun-WSGI: cannot send as provider not attached!")
            raise

        finally:
            if hasattr(iterable, 'close'):
                try:
                    iterable.close()
                except Exception as e:
                    if self.sender:
                        try:
                            rg_request_details = http_utilities.build_wsgi_compliant_request(
                                environ)
                            self.sender.send_exception(
                                exception=e, request=rg_request_details)
                        except Exception as inner:
                            log.error(
                                "Raygun-WSGI: could not send exception on close!")
                            log.error(inner)
                    else:
                        log.error(
                            "Raygun-WSGI: cannot send on close as provider not attached!")
                    raise



from optparse import OptionParser

from raygun4py import raygunprovider


def main():
    usage = '\n  raygun4py test <apikey>'
    parser = OptionParser(usage=usage)

    _, args = parser.parse_args()

    if len(args) < 2 or not isinstance(args[1], str):
        print('Please provide your API key')
        parser.print_help()
    elif args[0] == 'test':
        send_test_exception(args[1])
    else:
        print(f"Invalid command '{args[0]}'")
        parser.print_help()


def send_test_exception(apikey):
    client = raygunprovider.RaygunSender(apikey)

    try:
        raise Exception("Test exception from Raygun4py (Python3)")
    except Exception:
        response = client.send_exception()

        if response[0] == 202:
            print("Success! Now check your Raygun dashboard at https://app.raygun.com")
        else:
            print(
                "Something went wrong - please check your API key or contact us for help. The response was:")
            print(response)



import sys

from setuptools import setup

packages = ['raygun4py', 'raygun4py.middleware']

base_dir = 'python2'
if sys.version_info[0] == 3:
    base_dir = 'python3'

exec(open('%s/raygun4py/version.py' % base_dir).read())
requirements = [
    'jsonpickle >= 0.9.2',
    'blinker >= 1.3.0',
    'requests >= 2.9.1'
]

dev_requirements = [
    'unittest2 >= 1.1.0',
    'coveralls >= 1.5.1',
    'mock >= 2.0.0',
    'django >= 1.8.8',
    'flask >= 0.10',
    'WebTest >= 2.0.32'
]

setup(
    name='raygun4py',
    version=__version__,
    packages=packages,
    package_dir={
        "raygun4py": base_dir + "/raygun4py"
    },
    license='LICENSE',
    url='https://raygun.com',
    author='Raygun',
    author_email='hello@raygun.io',
    description='Official Raygun provider for Python 2.7 and Python 3+',
    long_description=open('README.rst').read(),
    install_requires=requirements,
    extras_require={
        'dev': dev_requirements
    },
    entry_points={
        'console_scripts': [
            'raygun4py = raygun4py.cli:main'
        ]
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Topic :: Communications',
    ],
)



import logging

from raygun4py import raygunprovider

API_KEY = "paste_your_api_key_here"

# Initialize a RaygunHandler from an existing RaygunSender
sender = raygunprovider.RaygunSender(API_KEY)
sender.set_version("1.3")
sender.set_user({
    'identifier': 'example@email_or_user_id.com',
    'firstName': 'John',
    'fullName': 'John Smith',
    'email': 'example@email_or_user_id.com'
})
logging_handler_from_sender = raygunprovider.RaygunHandler.from_sender(
    sender, level=logging.INFO)

# Initialize a RaygunHandler using an api_key
logging_handler_from_key = raygunprovider.RaygunHandler(
    API_KEY, level=logging.INFO)

logger = logging.getLogger()
logger.setLevel(logging.INFO)
logger.addHandler(logging_handler_from_sender)
logger.addHandler(logging_handler_from_key)


def demonstrate_logging():
    # Sending an INFO log (not recommended)
    logger.info("Test INFO log sent from raygun4py")

    # Sending an ERROR log with exc_info=True
    try:
        raise ValueError("This is a test ValueError")
    except ValueError:
        logger.error(
            "Test ERROR log with exc_info=True sent from raygun4py", exc_info=True)

    # Sending an ERROR log without exc_info
    logger.error("Test ERROR log sent from raygun4py")

    # Using logger.exception which automatically includes exc_info
    try:
        raise RuntimeError("This is a test RuntimeError")
    except RuntimeError:
        logger.exception("Test logger.exception log sent from raygun4py")


if __name__ == '__main__':
    demonstrate_logging()



import sys

from raygun4py import raygunprovider

API_KEY = "paste_your_api_key_here"

sender = raygunprovider.RaygunSender(API_KEY)
sender.set_version("1.3")
sender.set_user({
    'identifier': 'example@email_or_user_id.com',
    'firstName': 'John',
    'fullName': 'John Smith',
    'email': 'example@email_or_user_id.com'
})


def handle_exception(exc_type, exc_value, exc_traceback):
    headers = {
        "referer": "localhost",
        "user-Agent": "Mozilla"
    }
    request = {
        "headers": headers,
        "hostName": "localhost",
        "url": "/resource",
        "httpMethod": "GET",
        "ipAddress": "127.0.0.1",
        "queryString": None,
        "form": None,
        "rawData": None,
    }
    sender.send_exception(exc_info=(exc_type, exc_value, exc_traceback), tags=[
                          "tag1", "tag2"], userCustomData={"key1": 1111, "key2": 2222}, request=request)
    sys.__excepthook__(exc_type, exc_value, exc_traceback)


def very_buggy_request():
    try:
        raise Exception("Test caught exception sent from raygun4py")
    except:
        sender.send_exception()

    methodtwo()


def methodtwo():
    raise Exception("Test uncaught exception sent from raygun4py")


sys.excepthook = handle_exception

very_buggy_request()
